//imclient.h

#ifndef _imclient_h_
#define _imclient_h_
#include <unistd.h>
#include <sys/mman.h>
#include <semaphore.h>
#include <fcntl.h>
#include <sys/stat.h>

/*
	17.01.2018
*/

typedef struct _SSHM
{
	u_int64_t	SMOS;	//размер выделяемой памяти
	int 		shm;
	sem_t 		*sem;
} SSHM;

/*формирование разделяемой памяти для режима SHM
SMON	- имя файла памяти в /dev/shm
SN	- имя семафора в /dev/shm
SMOS	- размер выделяемой памяти
*/
__inline int _SshMemCreate(char * SMON, char * SN, u_int64_t SMOS, SSHM * sSHM)
{
	int res = 0;
	//память - создание
	if ( (sSHM->shm = shm_open(SMON, O_CREAT|O_RDWR, S_IRWXO|S_IRWXG|S_IRWXU)) == -1 )
        	return 1;
	if ( ftruncate(sSHM->shm, SMOS + 1) == -1 ) 
		return 2;
	sSHM->SMOS	= SMOS;
 	//семафор - создание
	if ( (sSHM->sem = sem_open(SN, O_CREAT, 0777, 0)) == SEM_FAILED ) 
		return 3;
	sem_post(sSHM->sem);
	//
	return res;
}
#define _SshMemOpen	_SshMemCreate

/*
чтение блока из разделяемой памяти
*/
__inline int _SshMemBlockRead(u_int8_t * bl, u_int64_t raddr, u_int64_t size, SSHM * sSHM)
{
	u_int8_t	*addr;
	sem_wait(sSHM->sem);
	if ( (addr = mmap(0, sSHM->SMOS+1, PROT_WRITE|PROT_READ, MAP_SHARED, sSHM->shm, 0)) == (u_int8_t*)-1 )
       		return 1;
	//проверка размера
	if(size > sSHM->SMOS)	
		return 2;
	memcpy(bl, addr + raddr, size);
	sem_post(sSHM->sem);
	return 0;
}

/*
удаление разделяемой памяти и семафора для режима SHM
*/
__inline int _SshMemRemove(char * SMON, char * SN)
{
	shm_unlink(SMON);
	sem_unlink(SN);
	return 0;
}

/*
запись блока в разделяемую память
*/
__inline int _SshMemBlockWriete(u_int8_t * bl, u_int64_t raddr, u_int64_t size, SSHM * sSHM)
{
	u_int8_t	*addr;
	sem_wait(sSHM->sem);
	if ( (addr = mmap(0, sSHM->SMOS+1, PROT_WRITE|PROT_READ, MAP_SHARED, sSHM->shm, 0)) == (u_int8_t*)-1 )
       		return 1;
	//проверка размера
	if(size > sSHM->SMOS)	
		return 2;
	memcpy(addr + raddr, bl, size);
	sem_post(sSHM->sem);
	return 0;
}


/*
секция для "клиента" - "конечной точки"
*/

typedef struct _CLESHM
{
	SSHM		din;	//данные из конечной точки
	SSHM		dout;	//данные в конечную точку
} CLESHM;



#endif

